# HCOSMED - Compute median of input images for cosmic ray removal
# May 11, 2007
# By Doug Mink, SAO Telescope Data Center

procedure hcosmed (images)

string	images=""	{prompt="Images to combine"}
file	medfile=""	{prompt="Name of output medianfile"}
int	nfiles=0	{prompt="Number of images (count if 0)"}
string	combine="median" {prompt="Combine images (average|median|sum)"}
string	reject="crreject"	{prompt="Reject method"}
bool	deltemp=no	{prompt="Delete intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin
	int lroot
	string imagelist, newtitle, combim, rejim, outroot
	file imin, templist, imfile
	bool vb

	vb = verbose
	imagelist = images
	rejim = reject
	combim = combine

#  Make temporary list of input files
	templist = mktemp ("hmedlist")
	files (imagelist, >templist)

# Count files and initialize output root name
	if (nfiles < 1) {
	    nfiles = 0
	    list = templist
	    while (fscan (list,imin) != EOF) {
		smatch (imin, ".fits")
		if (smatch.found) {
		    imfile = imin
		    }
		else {
		    imfile = imin // ".fits"
		    }
		if (!access (imfile)) {
		    if (vb) {
			print ("HCOSMED: Image ",imfile," not found")
			}
		    next
		    }
		nfiles = nfiles + 1
		if (medfile == "") {
		    smatch (imin, ".fits")
		    if (smatch.found) {
			lroot = smatch.start - 6
			outroot = substr (imin, 1, lroot)
			}
		    else {
			lroot = strlen (imin) - 5
			outroot = substr (imin, 1, lroot)
			}
		    medfile = outroot // "_med.fits"
		    }
		}
	    }
	if (medfile == "") {
	    list = templist
	    if (fscan (list,imin) != EOF) {
		smatch (imin, ".fits")
	    	if (smatch.found) {
		    lroot = smatch.start - 6
		    outroot = substr (imin, 1, lroot)
		    }
		else {
		    lroot = strlen (imin) - 5
		    outroot = substr (imin, 1, lroot)
		    }
		}
	    medfile = outroot // "_med.fits"
	    }
	delete (templist)
	if (nfiles == 0) {
	    print ("No files to process")
	    return
	    }
	if (access (medfile)) {
	    if (vb) {
		print ("HCOSMED: deleting existing median file ",medfile, ".")
		}
	    delete (medfile)
	    }
	if (vb) {
	    print ("HCOSMED: median of ",nfiles," files is ",medfile)
	    }

# Initialize imcombine
	imcombine.headers = ""
	imcombine.bpmasks = ""
	imcombine.nrejmasks = ""
	imcombine.expmasks = ""
	imcombine.sigmas = ""
	imcombine.logfile = "STDOUT"
	imcombine.project = no
	imcombine.outlimits = ""
	imcombine.offsets = "none"
	imcombine.masktype = "none"
	imcombine.maskvalue = 0.
	imcombine.blank = 0.
	imcombine.scale = "none"
	imcombine.zero = "none"
	imcombine.weight = "none"
	imcombine.statsec = ""
	imcombine.expname = ""
	imcombine.lthreshold = INDEF
	imcombine.hthreshold = INDEF
	imcombine.nlow = 1
	imcombine.nhigh = 1
	imcombine.nkeep = 1
	imcombine.mclip = yes
	imcombine.lsigma = 3.
	imcombine.hsigma = 3.
	imcombine.rdnoise = "0."
	imcombine.gain = "1."
	imcombine.snoise = "0."
	imcombine.sigscale = 0.1
	imcombine.pclip = -0.5
	imcombine.grow = 0.

#  Make median from input images
	imcombine.combine = combim
	imcombine.reject = rejim
	imcombine.outtype="real"
	imcombine.rejmasks = ""
	imcombine (imagelist, medfile)
	imgetp (medfile,"i_title")
	newtitle = imgetp.value // " median"
	hedit (medfile, "i_title", newtitle)

	if (vb) {
	    print ("HCOSMED: Median image ",medfile," from ",imagelist)
	    }
	return
end

# Apr 17 2007	New task based on Nelson Caldwell's Perl script
# May  4 2007	Add recompute parameter to optionally use old files
# May  9 2007	Fix program name on messages
# May 10 2007	Add type of image to OBJECT keyword value
# May 11 2007	Add outroot and nfiles parameters to cosmed()
