# File t_hcosmic.x
# May 17, 2007
# By Doug Mink after Nelson Caldwell

include <error.h>
include <ctype.h>
include <imhdr.h>
include <imio.h>
include <mach.h>

define	MAXROWL		4096
define	MAXLINE		4608
define	MAXBUFF		100000

# HCOSMIC -- Locate cosmic rays in a pair of images by looking at
# subsections for which IRAF has already determined the sigma
# Output map file(s) where pixels where the image has a value
# over the detection limit (parameter "value") are set to 3

procedure t_hcosmic ()

pointer	im1, im2, im3, im4, im5, pdata
real	data1[MAXROWL,MAXLINE],data2[MAXROWL,MAXLINE]
real	datam[MAXROWL,MAXLINE]
short	data3[MAXROWL,MAXLINE],data4[MAXROWL,MAXLINE]
real	datat[MAXBUFF],datat2[MAXBUFF]
char	name1[SZ_FNAME]
char	name2[SZ_FNAME]
char	nameo[SZ_FNAME]
char	nameo2[SZ_FNAME]
char	med[SZ_FNAME]
long	v[IM_MAXDIM]
bool	verbose, proc2
int	nc, nl, kkk, ix, ix2, iy, iy2, ij, i, j, ijj, kk, jj
int	nbad1, nbad2, npix, ijmax
short	linemin, linemax
real	ratio, value, stat, stat1, stat2, diff, diff2, xsum1, xsum2
pointer	outline
pointer	imgl2r(), impl2s(),immap()
real	clgetr()
int	impnls()
bool	clgetb()
long	clktime()

begin

# Get parameters
	call clgstr ("image1", name1, SZ_FNAME)
	call clgstr ("image2", name2, SZ_FNAME)
	call clgstr ("imout", nameo, SZ_FNAME)
	call clgstr ("imout2", nameo2, SZ_FNAME)
	call clgstr ("limfile", med, SZ_FNAME)
	proc2 = clgetb ("proc2")
	value = clgetr ("value")
	verbose = clgetb ("verbose")

# Open the first image
	iferr (im1 = immap (name1, READ_ONLY, 0)) {
	    call printf ("HCOSMIC: Cannot read %s\n")
		call pargstr (name1)
	    call clputi ("error", 1)
	    return
	    }
	nc = IM_LEN(im1,1)
	nl = IM_LEN(im1,2)
	nbad1 = 0
	nbad2 = 0
	if (verbose) {
	    call printf ("HCOSMIC: %d x %d x %d input image %s opened\n")
		call pargi (IM_LEN(im1,1))
		call pargi (IM_LEN(im1,2))
		call pargi (IM_PIXTYPE(im1))
		call pargstr (name1)
	    call flush (STDOUT)
	    }

# Read the first input image
	do i = 1, nl {
	    pdata = imgl2r (im1, i)
	    do kkk = 1, nc {
		data1[kkk,i] = Memr[pdata + kkk - 1]
		}
	    }
	if (verbose) {
	    call printf ("HCOSMIC: Input image %s read\n")
		call pargstr (name1)
	    call flush (STDOUT)
	    }

# Open the second image
	iferr (im2 = immap (name2, READ_ONLY, 0)) {
	    call printf ("HCOSMIC: Cannot read %s\n")
		call pargstr (name2)
	    call clputi ("error", 2)
	    return
	    }

	if (verbose) {
	    call printf ("HCOSMIC: %d x %d x %d input image %s opened\n")
		call pargi (IM_LEN(im2,1))
		call pargi (IM_LEN(im2,2))
		call pargi (IM_PIXTYPE(im2))
		call pargstr (name2)
	    call flush (STDOUT)
	    }

# Read the second image
	do i = 1, nl {
	    pdata = imgl2r (im2, i)
	    do kkk = 1, nc {
		data2[kkk,i] = Memr[pdata + kkk - 1]
		}
	    }
	if (verbose) {
	    call printf ("HCOSMIC: Input image %s read\n")
		call pargstr (name2)
	    call flush (STDOUT)
	    }

# Open the variance image
	iferr (im3 = immap (med, READ_ONLY, 0)) {
	    call printf ("HCOSMIC: Cannot read %s\n")
		call pargstr (med)
	    call clputi ("error", 3)
	    return
	    }

	if (verbose) {
	    call printf ("HCOSMIC: Variance image %s opened\n")
		call pargstr (med)
	    call flush (STDOUT)
	    }

# Read the variance image
	do i = 1, nl {
	    pdata = imgl2r (im3, i)
	    do kkk = 1, nc {
		datam[kkk,i] = Memr[pdata + kkk - 1]
		}
	    }

# Close the variance image
	call imunmap (im3)

	if (verbose) {
	    call printf ("HCOSMIC: Variance image %s read\n")
		call pargstr (med)
	    call flush (STDOUT)
	    }

# Compare sections 3 pixels wide by 20 pixels long
 	for (ix = 1; ix <= nc; ix = ix + 3) {
	    ix2 = ix + 2
	    if (ix2 > nc)
		ix2 = nc

 	    for (iy = 1; iy <= nl; iy = iy + 20) {
		iy2 = iy + 19
		if (iy2 > nl)
		    iy2 = nl

# Form scaling ratio from a box
		ij=0
		do i = ix, ix2 {
		    do j=iy , iy2 {
			ij = ij + 1
			datat2[ij] = data2[i,j]
			datat[ij] = data1[i,j]
			}
		    }

# Form a sort of median in both boxes to get the mean ratio of fluxes
		call asrtr (datat, datat, ij)
		call asrtr (datat2, datat2, ij)
		ijmax = 0.6 * ij
		xsum1 = 0
		xsum2 = 0
		do ijj = 1, ijmax {
		    xsum1 = xsum1 + datat[ijj]
		    xsum2 = xsum2 + datat2[ijj]
		    }

		stat1 = xsum1 / real (ijmax)
		stat2 = xsum2 / real (ijmax)
		if ((stat2 > 20.0) || (stat1 > 20.0))
		    ratio = stat1 / stat2

# For low signal cases
		else
		    ratio = 1.0

# Now look at pixels in that box
		do i = ix, ix2 {
		    do j = iy, iy2 {

# The statistical check for a cosmic is
#  pix1 - pix2*ratio > value*rms 
# where ratio is the flux ratio of image 1 to image 2 in a 2x20 box
# around the pixel, and value is set to 10
			stat = value * datam[i,j]
			data3[i,j] = 0
			diff = data1[i,j] - data2[i,j] * ratio
			diff2 = data2[i,j] - data1[i,j] / ratio

# 3 is the mask value for column interpolation
			if (diff > stat) {
			    data3[i,j] = 3
			    nbad1 = nbad1 + 1
			    }

# If doing two files at once
			if (proc2) {
			    data4[i,j] = 0
			    diff = data2[i,j] - data1[i,j] / ratio
			    if (diff > stat) {
				data4[i,j] = 3
				nbad2 = nbad2 + 1
				}
			    }
			}
		    }
		}
	    }

	if (verbose) {
	    call printf ("HCOSMIC: Found and marked %d bad pixels in %s\n")
		call pargi (nbad1)
		call pargstr (name1)
	    call flush (STDOUT)
	    if (proc2) {
		call printf ("HCOSMIC: Found and marked %d bad pixels in %s\n")
		    call pargi (nbad2)
		    call pargstr (name2)
		call flush (STDOUT)
		}
	    }

# Create the first or only output bad pixel image
# It is the same as the first input image except that it is short integer
	iferr (im4 = immap (nameo, NEW_COPY, im1)) {
	    call printf ("HCOSMIC: Cannot create %s\n")
		call pargstr (nameo)
	    call clputi ("error", 4)
	    return
	    }
	IM_PIXTYPE(im4) = TY_SHORT
	call imastr (im4, "i_title", "Cosmic ray map")
	call imastr (im4, "HCOSMIC", "Cosmic ray pixels marked with 3")
	if (verbose) {
	    call printf ("HCOSMIC: Opened %d x %d bad pixel image %s\n")
		call pargi (nc)
		call pargi (nl)
		call pargstr (nameo)
	    call flush (STDOUT)
	    }

 
# Write out the first bad pixel file one line at a time
	call amovkl (long(1), v, IM_MAXDIM)
	do kk = 1, nl {

# Get buffer for this line of image
	    v[2] = kk
	    if (impnls (im4, outline, v) == EOF) {
		call printf ("HCOSMIC: Error writing line %d of %s\n")
		    call pargi (kk)
		    call pargstr (nameo)
		call flush (STDOUT)
		call clputi ("error", 6)
		return
		}
	    call achtss (data3[1,kk], Mems[outline], nc)
	    }
	IM_CTIME(im4) = clktime (long(0))
	if (verbose) {
	    call printf ("HCOSMIC: Wrote  %d x %d bad pixel image %s\n")
		call pargi (IM_LEN(im4,1))
		call pargi (IM_LEN(im4,2))
		call pargstr (nameo)
	    call flush (STDOUT)
	    }
	IM_UPDATE(im4) = YES
	call imunmap (im4)

	if (proc2) {

# On first pass through, create a second output file
	    iferr (im5 = immap (nameo2, NEW_COPY, im2)) {
		call printf ("HCOSMIC: Cannot create %s\n")
		    call pargstr (nameo2)
		call clputi ("error", 5)
		return
		}
	    IM_MIN(im5) = MAX_SHORT
	    IM_MAX(im5) = -MAX_SHORT
	    IM_PIXTYPE(im5) = TY_SHORT
	    call imastr (im5, "i_title", "Cosmic ray map")
	    call imastr (im5, "HCOSMIC", "Cosmic ray pixels marked with 3")
	    if (verbose) {
		call printf ("HCOSMIC: Opened %d x %d bad pixel image %s\n")
		    call pargi (nc)
		    call pargi (nl)
		    call pargstr (nameo2)
		call flush (STDOUT)
		}

# Write out the second bad pixel file one line at a time
	    call amovkl (long(1), v, IM_MAXDIM)
	    do kk = 1, nl {
		v[2] = kk
		if (impnls (im5, outline, v) == EOF) {
		    call printf ("HCOSMIC: Error writing line %d of %s\n")
			call pargi (kk)
			call pargstr (nameo2)
		    call flush (STDOUT)
		    call clputi ("error", 7)
		    return
		    }
		call achtss (data4[1,kk], Mems[outline], nc)
		}
	    IM_CTIME(im5) = clktime (long(0))
	    if (verbose) {
		call printf ("HCOSMIC: Wrote  %d x %d bad pixel file %s\n")
		    call pargi (IM_LEN(im5,1))
		    call pargi (IM_LEN(im5,2))
		    call pargstr (nameo2)
		call flush (STDOUT)
		}
	    IM_UPDATE(im5) = YES
	    call imunmap (im5)
	    }

# Close input images
	call imunmap (im1)
	call imunmap (im2)

end
# May 10 2007	New program
# May 17 2007	Drop top 40% of pixels, not just top 4, for statistics
