# HPREAMP - Run SPP version of Brian McCleod's preamplifier fix on raw images
# November 5, 2007
# By Doug Mink, SAO Telescope Data Center

procedure hpreamp (images)

string	images=""	{prompt="Images to process"}

bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	saveold=yes	{prompt="Save unprocessed image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (yes or no)"}
bool	debug=no	{prompt="Print more process progress (yes or no)"}

begin

file	imroot, temp1, imin, templist, imout
string	imx, ims, imsec, history, tchar
string	pfsect0, pfsect1, pfsect2, pfsect3, pfsect4
bool	vb, dt, db

	vb = verbose
	dt = deltemp
	db = debug
	imx = ".fits"
	ims = image

	if (ims == "") {
	    print ("HPREAMP: Correct preamp problem for all four amps one amp at a time")
	    print ("Usage: hpreamp <input images with 4 extensions>")
	    return
	    }

# Make directory in which to save raw multi-extension files
	if (saveold) {
	    if (!access ("Raw"))
		mkdir ("Raw")
	    }

#  Make temporary list of input files
	tchar = substr (ims,1,1)
	if (tchar == '@') {
	    templist = substr (ims,2,100)
	    deltemp = no
	    }
	else {
	    templist = mktemp ("proclist")
	    files (ims, >templist)
	    }
	list = templist
        while (fscan (list,imin) != EOF) {

# Do not process images without extensions
	    imgetp.default = "-1"
	    imsec = imin // "[0]"
	    imgetp (imsec, "i_naxis")
	    if (imgetp.value != "0") {
		print ("HPREAMP: ", imin, " is not a multi-extension image")
		print ("NAXIS = ", imgetp.value)
		next
		}

# Do not process image twice
	    imgetp.default = "0"
	    imsec = imin // "[1]"
	    imgetp (imsec, "PREAMPFX")
	    if (imgetp.value != "0") {
		if (vb) {
		    print ("HPREAMP: ", imin, " has already been processed")
		    print ("PREAMPFX = ", imgetp.value)
		    }
		next
		}

# Find root name of image
	    imroot = imin
	    smatch (imroot, imx)
	    if (smatch.found) {
		imroot = substr (imroot,1,smatch.start-1)
		}
	    else {
		imroot = imageout
		imin = imin // imx
		}

# Fix preamp output using SPP version of Brian McLeod's C program preampfix
	    if (db) {
		print ("HPREAMP: Making preamp correction for ",imin)
		print ("HPREAMP: image root is ",imroot)
		}
	    preamp1.verbose = vb
	    preamp1.debug = db
	    preamp1.error = ""
	    preamp1.indir = ""
	    pfsect0 = imroot // "_0.fits"
	    imcopy (imin // "[0]", pfsect0, verbose=vb)
	    pfsect1 = imroot // "_1.fits"
	    imcopy (imin // "[1]", pfsect1, verbose=vb)
	    preamp1 (pfsect1)
	    if (preamp1.error != "")
		print ("HPREAMP 1: ",error)
	    pfsect2 = imroot // "_2.fits"
	    imcopy (imin // "[2]", pfsect2, verbose=vb)
	    preamp1.error = ""
	    preamp1 (pfsect2)
	    if (preamp1.error != "")
		print ("HPREAMP 2: ",error)
	    pfsect3 = imroot // "_3.fits"
	    imcopy (imin // "[3]", pfsect3, verbose=vb)
	    preamp1.error = ""
	    preamp1 (pfsect3)
	    if (preamp1.error != "")
		print ("HPREAMP 3: ",error)
	    pfsect4 = imroot // "_4.fits"
	    imcopy (imin // "[4]", pfsect4, verbose=vb)
	    preamp1.error = ""
	    preamp1 (pfsect4)
	    if (preamp1.error != "")
		print ("HPREAMP 4: ",error)

# Save unchanged image
	    if (saveold) {
		imout = "Raw/" // imin
		if (!access (imout))
		    rename (imin, imout)
		else
		    delete (imin)
		}
	    else
		delete (imin)

# Reassemble the multi-extension FITS file
	    if(vb)
		print("HPREAMP: rejoining ",imin)

# Reassemble the input file with the fixed extensions
	    mscjoin (imroot, output = imin, delete=dt, verbose=vb)
	    }

	if (deltemp)
	    delete (templist)
end

# Oct 23 2007	New program to run SPP preamp1 instead of C program preampfix
