# HREPORT1 -- Report on Hectospec processing from keywords in image header
# By Doug Mink, SAO/TDC
# April 1, 2007

procedure hreport1 (image)

string  image = ""      {prompt="Input multispec image to correct"}
bool verbose = yes      {prompt="Log progress (y or n)"}
bool debug = no         {prompt="Add more information (y or n)"}

begin
	bool vb, db
	file imin, imroot
	int nap, npix, lroot, ncombine
	int ifile
	real wl1, wl2, dwl
	string swl1, swl2, sdwl, keyword, text

	imin = image
	vb = verbose
	db = debug

# Make sure filename ends in .ms.fits
	smatch (imin, ".ms")
	if (smatch.found) {
	    lroot = smatch.start - 1
	    smatch (imin, ".fits")
	    if (!smatch.found) {
		imin = imin // ".fits"
		}
	    }
	else {
	    smatch (imin, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imin = substr (imin, 1, lroot) // ".ms.fits"
		}
	    else {
		imin = imin // ".ms.fits"
		}
	    }

	if (!access (imin)) {
	    print ("HREPORT1: ", imin," does not exist")
	    return
	    }

# Print image name and dimensions
	imgetp.verbose = db
	imgetp.default = "0"
	imgetp (imin, "i_naxis1")
	npix = int (imgetp.value)
	imgetp (imin, "i_naxis2")
	nap = int (imgetp.value)
	imgetp (imin, "i_naxis2")
	nap = int (imgetp.value)
	imgetp (imin, "CRVAL1")
	wl1 = real (imgetp.value)
	round (wl1,"%7.2f")
	swl1 = round.value
	imgetp (imin, "CD1_1")
	if (imgetp.value == "0") {
	    imgetp (imin, "CDELT1")
	    }
	dwl = real (imgetp.value)
	round (dwl,"%4.2f")
	sdwl = round.value
	wl2 = wl1 + (dwl + (npix - 1))
	round (wl2,"%7.2f")
	swl2 = round.value
	print (imin,": ", nap,"aps x ",npix,"pix, ",swl1,"-",swl2, " by ",sdwl," A") 

# Print images combined
	imgetp.default = "0"
	imgetp (imin, "HSCOMBIN")
	if (imgetp.value != "0") {
	    print ("    ", imgetp.value)
	    }
	imgetp (imin, "NCOMBINE")
	ncombine = int (imgetp.value)
	for (ifile = 1; ifile <= ncombine; ifile = ifile + 1) {

#	Image file name
	    keyword = "HSCOMB" // ifile
	    imgetp (imin, keyword)
	    print (ifile,": ",imgetp.value)

#	Pixel shift
	    keyword = "HSHIFT" // ifile
	    imgetp (imin, keyword)
	    if (imgetp.value == "0") {
		imgetp (imin,"HPIXSHIF")
		}
	    print ("    ",imgetp.value)
	    }

# Linearization and interpolation
	    keyword = "HLIN" // ifile
	    imgetp (imin, keyword)
	    if (imgetp.value != "0") {
		print ("    ",imgetp.value)
		}
	    else {
		imgetp (imin,"HLIN")
		text = imgetp.value
		imgetp (imin,"INTERP")
		text = text // " " // imgetp.value
		print ("    ", text)
		}
	
# Print logged fiber color throughput correction
	imgetp.default = "0"
	imgetp (imin, "HTRAN")
	if (imgetp.value == "0") {
	    imgetp (imin, "FIBRTRAN")
	    }
	if (imgetp.value != "0") {
	    print (" HTRAN    = ", imgetp.value)
	    }

# Print logged sky absorption removal
	imgetp.default = "0"
	imgetp (imin, "HABSKY")
	if (imgetp.value != "0") {
	    print (" HABSKY   = ", imgetp.value)
	    }

# Print logged red leak removal
	imgetp.default = "0"
	imgetp (imin, "HREDLEAK")
	if (imgetp.value != "0") {
	    print (" HREDLEAK = ", imgetp.value)
	    }

end

# Apr  1 2007	New task
