# XCSKY - Piecewise cross-correlate a spectrum against a template in pixel space
#         using MKXCTEMPW and XCSAO, make new cal file id file, and reidentify
# November 4, 2003
# By Doug Mink, SAO Telescope Data Center

procedure xcsky (objects)

file	objects=""	{prompt="Object spectrum file"}
string	aprange="1"	{prompt="Object aperture"}
int	band=0		{prompt="Object band"}
file	objtemp = ""	{prompt="Template spectrum file"}
file	objsect = ""	{prompt="File of pixel sections to cross-correlate"}
file	caltemp	=""	{prompt="Template calibration file"}
file	calshift=""	{prompt="Shifted and stretched calibration file"}
int	nbin=256	{prompt="Number of points to cross-correlate (rebinned)"}
real	ndev=0		{prompt="Reject past this number of standard deviations"}
real	rmin=2.0	{prompt="Reject segment cross-correlations with R < this"}
char	xcmode="pixel"	{prompt="Cross-correlate in pixel, wavelength, or velocity"}
bool	intdisp=no	{prompt="Fit dispersion function interactively (y or n)"}
bool	intxcor=no	{prompt="Run xcsao interactively (y or n)"}
bool	debug=no	{prompt="Print debugging information (y or n)"}

begin
	char tchar
	char xmode
	file obj1
	file filelist, tempfile, templist
	string tempsect
	string obj, objfile
	int icom, icom1, lcom
	bool pixout

	xmode = xcmode
	tempfile = objtemp
	tempsect = objsect
	if (xmode == "pixel") {
 	    pixout = yes
	    }
	else {
	    pixout = no
	    } 
	xcsky1.objband = band
	xcsky1.objap = ap
	xcsky1.caltemp = caltemp
	xcsky1.calshift = calshift
	xcsky1.nbin = nbin
	xcsky1.intxcor = intxcor
	xcsky1.ndev = ndev
	xcsky1.rmin = rmin
	xcsky1.intdisp = intdisp
	xcsky1.xcmode = xcmode
	xcsky1.debug = debug

	obj = objects
	tchar = substr (obj,1,1)


# Make list of templates based on wavelength regions in file
        templist = mktemp ("xcsky")
        mkxctempw (tempfile,tempsect,templist=templist,pixout=pixout)

# Process single spectrum
	if (tchar != '@') {
	    icom = stridx (obj, ',')
	    if (icom < 1) {
		do i = 1, 300 {
		    getrange (aprange, i)
		    ap = getrange.ival
		    if (ap > 0) {
			xcsky1.objap = ap
			xcsky1 (obj, templist)
			}
		    else {
			break
			}
		    }
		}

# Process comma-separated list of spectrum files
	    else {
		icom1 = 1
		while (icom1 > 0) {
		    if (icom < 1) {
			lcom = 128
			}
		    else {
			lcom = icom - icom1 - 1
			}
		    obj1 = substr (obj, icom1, lcom)
		    do i = 1, 300 {
			getrange (aprange, i)
			ap = getrange.ival
			if (ap > 0) {
			    xcsky1.objap = ap
			    xcsky1 (obj, templist)
			    }
			else {
			    break
			    }
			}

		# Find next file name
		    icom1 = icom + 1
		    obj = substr (obj, icom1, 128)
		    if (icom < 1) {
			break
			}
		    else {
			icom = stridx (obj, ',')
			}
		    }
		}
	    }

# Process file listing spectrum files
	else {
	    filelist = substr (obj,2, 128)
	    print ("Reading files from ", filelist)
	    list = filelist
	    while (fscan (list, objfile) != EOF) {
		do i = 1, 300 {
		    getrange (aprange, i)
		    ap = getrange.ival
		    if (ap > 0) {
			xcsky1.objap = ap
			xcsky1 (obj, templist)
			}
		    else {
			break
			}
		    }
		}
	    }
end

# Jul 21 2003	New program to call xcsky1
# Oct 30 2003	Add option to cross-correlate in wavelenght or velocity, too.
# Nov  4 2003	Pass aperture as well as band
