#*** File tres/Src/t_preamp1.x
#*** October 23, 2007
#*** By Doug Mink

include	<mach.h>
include	<imhdr.h>
include	<imio.h>
include	<gset.h>

define  MAXCOL		4700
define  MAXLINE		4700
define	SAMP1		(27+75)/2.    # The middle of the 1st integration
define	SAMP2		(141+94)/2.   # The middle of the 2nd integration
define	RESET		1.            # The start of the reset
define	SW		79.           # The start of SW pulse
define	PIXELTICKS	153.          # Number of ticks for reading one pixel

define	TICKTIME         62.5e-9
define	RC		(332e3 * 6800e-12) # Hectos preamp time constant
					   # as per JG email  4/29/04

# T_PREAMP1 -- Fix one TRES preamp

procedure t_preamp1

int	i, j, nc, nr
pointer im, sp, inbuf, outbuf
int	datafix[MAXCOL,MAXLINE]
char	imin[SZ_FNAME]
char	inpath[SZ_PATHNAME]
char    hstring[SZ_LINE]
char    dstring[SZ_LINE]
char	errlin[SZ_LINE]
int	lpath, pxtype, nfix
bool	verbose, debug, clgetb()
double	v0, epsilon, samp1, samp2, c1, c2, c3, bzero, bscale, s, value
long	v[IM_MAXDIM]

int	clgeti(), strlen()
long	clktime()
pointer	immap(), imgl2s(), impnls()
errchk	immap()

begin
	verbose = clgetb ("verbose")
	debug = clgetb ("debug")

	# Get the image input name.
	call clgstr ("infile", imin, SZ_LINE)
	call clgstr ("indir", inpath, SZ_LINE)
	
	lpath = strlen (inpath)
	if (lpath > 0) {
	    if (inpath[lpath] != '/') {
		inpath[lpath] = '/'
		inpath[lpath+1] = EOS
		}
	    call strcat (imin, inpath, SZ_FNAME)
	    }
	else
	    call strcpy (imin, inpath, SZ_FNAME)

	iferr (im = immap (inpath, READ_WRITE, 0)) {
	    call sprintf (errlin, SZ_LINE, "PREAMP1: Cannot read and write file %s")
		call pargstr (imin)
	    call printf ("%s\n")
		call pargstr (errlin)
	    call flush (STDOUT)
	    call clpstr ("error", errlin)
	    return
	    }

	# Get image dimensions
	nc = IM_LEN(im,1)
	nr = IM_LEN(im,2)
	if (debug) {
	    call printf ("PREAMP1: %d x %d input image %s opened\n")
		call pargi (nc)
		call pargi (nr)
		call pargstr (imin)
	    call flush (STDOUT)
	    }

	hstring[1] = EOS
	call imgspar (im, "PREAMPFX", hstring, SZ_LINE)
	if (strlen (hstring) > 0) {
	    call printf ("PREAMP1: %s has already been processed\n")
		call pargstr (imin)
	    call printf ("PREAMPFX= %s\n")
		call pargstr (hstring)
	    return
	    }

	# Get image scaling
	bscale = 1.0
	call imgrpar (im, "BSCALE", bscale)
	bzero = 0.0
	call imgrpar (im, "BZERO", bzero)
	pxtype = IM_PIXTYPE(im)
	if (debug) {
	    call printf ("PREAMP1: bzero = %g, bscale = %g, pixtype = %d\n")
		call pargr (bzero)
		call pargr (bscale)
		call pargi (pxtype)
	    call flush (STDOUT)
	    }

	# Skip if not raw 16-bit image
	if (pxtype != TY_SHORT && pxtype != TY_USHORT) {
	    call eprintf ("PREAMP1: Invalid pixel length of %d\n")
	    if (pxtype == TY_LONG || pxtype == TY_INT)
		call pargi (32)
	    else if (pxtype == TY_REAL)
		call pargi (32)
	    else if (pxtype == TY_DOUBLE)
		call pargi (64)
	    else
		call pargi (0)
	    return
	    }
	if (debug) {
	    call printf ("PREAMP1: Ready to set constants\n")
	    call flush (STDOUT)
	    }

	# Make correction
	v0 = 0.d0
	epsilon = TICKTIME / RC		# 1/tau

#	epsilon = 62.5d-9 / (332.0d+3 * 6800.0d-12)
#	samp1 = (27.d0 + 75.d0) / 2.d0
#	samp2 = (141.d0 + 94.d0) / 2.d0
#	c1 = (samp2 - samp1) * epsilon
#	c2 = 1.d0 - (153.d0 * epsilon)
#	c3 = (79.d0 - 1.d0) * epsilon
 
	if (debug) {
	    call printf ("PREAMP1: Ready to correct pixel values\n")
	    call flush (STDOUT)
	    }
	nfix = 0
	do j = 1, nr {
	    inbuf = imgl2s (im, j)
	    call achtui (Mems[inbuf], datafix[1,j], nc)
	    do i = 1, nc {
		s = double (datafix[i,j])
#		Ticks(SAMP1,SAMP2)  = (SAMP2 = SAMP1)
		value = (SAMP2 - SAMP1) * epsilon * v0
#		value = c1 * v0
		s = s + value + 1.d0
		datafix[i,j] = int (s)
#		if (s > 0)
#		    datafix[i,j] = int (s + 0.5)
#		else if (s < 0)
#		    datafix[i,j] = int (s - 0.5)
#		else
#		    datafix[i,j] = 0
#		Ticks(SW, RESET) - Ticks (79., 1.) = RESET - SW + PIXELTICKS
		v0 = v0 * ( 1 - PIXELTICKS * epsilon) - (RESET - SW + PIXELTICKS) * epsilon * s
#		v0 = (v0 * c2) - (c3 * s)
		}
	    nfix = nfix + nc
	    }
	if (debug) {
	    call printf ("PREAMP1: %d pixel values corrected\n")
		call pargi (nfix)
	    call flush (STDOUT)
	    }

	if (debug) {
	    call printf ("PREAMP1: writing %d x %d output fixed image %s\n")
		call pargi (nc)
		call pargi (nr)
		call pargstr (imin)
	    call flush (STDOUT)
	    }

# Initialize vector for output
	call amovkl (long(1), v, IM_MAXDIM)

# Write the output file one line at a time
	do j = 1, nr {

# Write one line at a time back to image
	    v[2] = j
	    if (impnls (im, outbuf, v) < nc) {
		call sprintf (errlin,SZ_LINE,"PREAMP1: Cannot write line %d of %s")
		    call pargi (j)
		    call pargstr (imin)
		call printf ("%s\n")
		    call pargstr (errlin)
		call flush (STDOUT)
		call clpstr ("error", errlin)
		return
	    }
	    call achtiu (datafix[1,j], Mems[outbuf], nc)
	    }

	# Put date into HISTORY
	call logtime (dstring, SZ_LINE)
	call sprintf (hstring,SZ_LINE,"%s Fixed by PREAMP1")
	    call pargstr (dstring)
	if (debug) {
	    call printf ("PREAMPFX= %s\n")
		call pargstr (hstring)
	    call flush (STDOUT)
	    }
	call imastr (im, "PREAMPFX", hstring)

# Close output image
	IM_CTIME(im) = clktime (long(0))
	IM_UPDATE(im) = YES
	if (debug) {
	    call printf ("PREAMP1: Ready to write output image %s\n")
		call pargstr (imin)
	    call flush (STDOUT)
	    }
	call imunmap (im)
	if (verbose) {
	    call printf ("PREAMP1: Preamp-fixed image %s written.\n")
		call pargstr (imin)
	    call flush (STDOUT)
	    }
	errlin[1] = EOS
	call clpstr ("ERROR", errlin)

end

# Oct 23 2007	New program, based on preampfix.c
